/*
 * This Source Code is subject to the terms of the Mozilla Public License
 * version 2.0 (the "License"). You can obtain a copy of the License at
 * http://mozilla.org/MPL/2.0/.
 */

Cu.import("resource://gre/modules/Services.jsm");

let {Prefs} = require("prefs");
Prefs.addListener(onPrefChange);
window.addEventListener("unload", function() Prefs.removeListener(onPrefChange), false);

function initCheckboxes()
{
	E("domainOptIn").checked = Prefs.domainOptIn;
}

function onItemSelected(list)
{
	let button = E(list.getAttribute("_removeButton"));
	let items = list.selectedItems;
	button.disabled = (items.length == 0 || (items.length == 1 && !items[0].value))
}

function onFindChange()
{
	let find = E("find");
	let button = E("add-button");
	button.disabled = !find.value;
}

function onPrefChange(name)
{
	if (name == "custom_replace" || name == "whitelist")
		updateList();
}

function updateList()
{
	//
	// Custom rules
	//

	let ruleListElement = E("custom_corrections");

	// Remove existing list entries
	for (let i = ruleListElement.getRowCount() - 1; i >= 0; i--)
		ruleListElement.removeItemAt(i);

	// Build a list of custom rules and sort it alphabetically
	let prefRulesList = Prefs.custom_replace;
	let ruleList = [];

	for (let searchString in prefRulesList)
	{
		ruleList.push([searchString, prefRulesList[searchString]]);
	}

	ruleList.sort(function(a, b)
	{
		if (a[0] < b[0])
			return -1;
		else if (a[0] > b[0])
			return 1;
		else
			return 0;
	});

	// Add the rules to the list
	if (ruleList.length > 0)
	{
		for (let i = 0; i < ruleList.length; i++)
		{
			let [searchString, replacement] = ruleList[i];

			let option = document.createElement('listitem');
			option.setAttribute("value", searchString);

			let cell1 = document.createElement('listcell');
			cell1.setAttribute("label", searchString);
			option.appendChild(cell1);

			let cell2 = document.createElement('listcell');
			cell2.setAttribute("label", replacement);
			option.appendChild(cell2);

			ruleListElement.appendChild(option);
		}
	}
	else
	{
		let option = document.createElement("listitem");
		option.setAttribute("class", "auto-entry");
		option.setAttribute("label", ruleListElement.getAttribute("_emptyLabel"));

		ruleListElement.appendChild(option);
	}

	//
	// Exception rules
	//

	let whitelistElement = E("whitelist");

	// Remove existing list entries
	for (let i = whitelistElement.getRowCount() - 1; i >= 0; i--)
		whitelistElement.removeItemAt(i);

	// Build a list of exceptions and sort it alphabetically
	let whitelist = Object.keys(Prefs.whitelist);
	whitelist.sort();

	// Add the rules to the list
	if (whitelist.length > 0)
	{
		for (let i = 0; i < whitelist.length; i++)
		{
			let option = document.createElement("listitem");
			option.setAttribute("value", whitelist[i]);
			option.setAttribute("label", whitelist[i]);

			whitelistElement.appendChild(option);
		}
	}
	else
	{
		let option = document.createElement("listitem");
		option.setAttribute("class", "auto-entry");
		option.setAttribute("label", whitelistElement.getAttribute("_emptyLabel"));

		whitelistElement.appendChild(option);
	}
}

function addRule()
{
	let searchString = E("find").value.trim();
	let replacement = E("replace").value.trim();
	if (searchString.length == 0)
		return;

	Prefs.custom_replace[searchString] = replacement;
	Prefs.custom_replace = JSON.parse(JSON.stringify(Prefs.custom_replace));

	E("find").value = E("replace").value = "";
	E("find").oninput();
}

function removeRule(btn, pref)
{
	let list = E(btn.getAttribute("_list"));
	let items = list.selectedItems;

	let {onWhitelistEntryRemoved} = require("rules");

	for (let i = items.length - 1; i >= 0; i--)
	{
		let searchString = items[i].getAttribute("value");
		delete Prefs[pref][searchString];

		if (pref == "whitelist")
			onWhitelistEntryRemoved(searchString);
	}
	Prefs[pref] = JSON.parse(JSON.stringify(Prefs[pref]));
}
