/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/FileUtils.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

// Import prefs from old branch if there are any
let {Prefs} = require("prefs");
Prefs.migrate("extensions.typed-it-collection.domainOptInAsk", "domainOptInAsk");
Prefs.migrate("extensions.typed-it-collection.domainOptIn", "domainOptIn");
Prefs.migrate("extensions.typed-it-collection.counter", "counter");

require("typoFixer");
require("typedItCollector");

// Execute first-run actions once the session is restored
{
	let observer =
	{
		observe: function(subject, topic, data)
		{
			removeHandler();
			onShutdown.remove(removeHandler);
			onBrowserInitialized();
		},
		QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver, Ci.nsISupportsWeakReference])
	};

	Services.obs.addObserver(observer, "sessionstore-windows-restored", true);

	// Maybe we were started mid-session, don't wait more than five seconds
	let timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
	timer.init(observer, 5000, Ci.nsITimer.TYPE_ONE_SHOT);

	let removeHandler = function()
	{
		Services.obs.removeObserver(observer, "sessionstore-windows-restored");
		timer.cancel();
	};
	onShutdown.add(removeHandler);
}

function onBrowserInitialized()
{
	function isMajorUpdate(version1, version2)
	{
		if (version1 == version2)
			return false;

		let oldParts = version1.split(".");
		let newParts = version2.split(".");
		return (newParts[0] != oldParts[0] || newParts[1] != oldParts[1]);
	}

	let window = Services.wm.getMostRecentWindow("navigator:browser");
	if (!window)
		return;

	let oldVersion = Prefs.version;
	Prefs.version = require("info").addonVersion;
	if (!oldVersion || isMajorUpdate(oldVersion, Prefs.version))
	{
		let url = "http://urlfixer.org/firstrun/?v=" + Prefs.version;
		if (oldVersion)
			url += "&update=1";
		if ("Browser" in window && typeof window.Browser.addTab != 'undefined')
		{
			// window.Browser.addTab(url, true);
			// No firstrun on Fennec.
		}
		else if ("gBrowser" in window)
			window.gBrowser.loadOneTab(url, {inBackground: false});
	}

	require("typedItCollector").onBrowserInitialized(window);
}
