/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

Cu.import("resource://gre/modules/Services.jsm");

let {Prefs} = require("prefs");

let surveyLang = null;
let surveyUrl = null; //"http://urlfixer.org/usersurvey/...";

exports.incrementCorrectionsCounter = incrementCorrectionsCounter;
function incrementCorrectionsCounter()
{
	// Only if survey exists
	if (!surveyUrl)
		return;
	
	// Only users with 5 URL corrections
	if (++Prefs.corrections_count == 5)
	{
		initSurvey();
	}
}

function initSurvey()
{
	// Don't ask after 2012-10-15
	if (Date.now() > 1350259200000)
		return;

	// Only Firefox users
	let {application} = require("info");
	if (application != "firefox")
		return;

	// Only Firefox 4 and higher
	if (Services.vc.compare(Services.appinfo.platformVersion, "2.0") < 0)
		return;

	// Survey is only available in English/German/Russian
	if (!/^(en|de|ru)\b/.test(Services.locale.getApplicationLocale().getCategory("NSILOCALE_CTYPE")))
		return;
	surveyLang = RegExp.$1;

	// Delay survey question by 5 seconds
	surveyTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
	surveyTimer.initWithCallback(runSurvey, 5000, Ci.nsITimer.TYPE_ONE_SHOT);
}

function runSurvey()
{
	//open panel
	let request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
	request.open("GET", "chrome://url-fixer/content/survey.xul");
	request.addEventListener("load", function(event)
	{
		let window = Services.wm.getMostRecentWindow("navigator:browser");
		let document = window.document;

		let style = document.createProcessingInstruction('xml-stylesheet', 'href="chrome://url-fixer/skin/survey.css" type="text/css"');
		document.insertBefore(style, document.firstChild);

		let panel = new window.DOMParser().parseFromString(request.responseText, "text/xml").documentElement;
		let oldPanel = document.getElementById(panel.id);
		if (oldPanel)
			oldPanel.parentNode.removeChild(oldPanel);
		document.getElementById("mainPopupSet").appendChild(panel);
		document.getElementById("url-fixer-icon").setAttribute("src", require("info").addonRoot + "icon64.png");
		document.getElementById("url-fixer-accept-button").addEventListener("command", function()
		{
			openSurvey();
			panel.hidePopup();
		});
		document.getElementById("url-fixer-cancel-button").addEventListener("command", function()
		{
			panel.hidePopup();
		});
		panel.addEventListener("popuphidden", function()
		{
			panel.parentNode.removeChild(panel);
			style.parentNode.removeChild(style);
		});

		let anchor = document.getElementById("identity-box");
		panel.openPopup(anchor, "after_start", 0, 0, false, true);
	}, false);
	request.send(null);
}

function openSurvey()
{
	let window = Services.wm.getMostRecentWindow("navigator:browser");
	if (window)
	{
		let browser = require("appIntegration").getBrowser(window);
		browser.loadOneTab(surveyUrl + "&lang=" + surveyLang, {
				referrerURI: Services.io.newURI("http://url.fixer/", null, null),
				inBackground: false
		});
	}
}
